local function get_bbox(glyph)
	local ulx,uly,lrx,lry = glyph[1], glyph[2], glyph[1], glyph[2]
	for i = 3,#glyph,2 do
		ulx = math.min(ulx, glyph[i])
		uly = math.min(uly, glyph[i+1])
		lrx = math.max(lrx, glyph[i])
		lry = math.max(lry, glyph[i+1])
	end
	return ulx,uly, lrx-ulx, lry-uly
end

GLYPHS = {}
next_glyph = coroutine.wrap(function()
	for char in (arg[2] or "ABCDEFGHIJKLMNOPQRSTUVWXYZ!&"):gmatch(".") do
		local vertices = {}
		coroutine.yield(vertices, char)

		-- scale vertices
		local x,y,w,h = get_bbox(vertices)
		local s = math.max(w,h)
		for i = 1,#vertices,2 do
			vertices[i]   = (vertices[i]   - x) / s
			vertices[i+1] = (vertices[i+1] - y) / s
		end

		local glyph = {vertices = vertices, width = w/s, height = h/s}

		function glyph:draw(x,y,size,jitter,times)
			local jitter = jitter or 0
			local times = times or 1
			for k = 1,times do
				local x = x + (math.random() * .2 - .1) * jitter
				local y = y + (math.random() * .2 - .1) * jitter
				for i=1,#self.vertices,4 do
					local x1,y1,x2,y2 = self.vertices[i], self.vertices[i+1], self.vertices[i+2], self.vertices[i+3]
					x1 = x1 + (math.random() * .2 - .1) * jitter
					y1 = y1 + (math.random() * .2 - .1) * jitter
					x2 = x2 + (math.random() * .2 - .1) * jitter
					y2 = y2 + (math.random() * .2 - .1) * jitter
					love.graphics.line(x1 * size + x, y1 * size + y, x2 * size + x, y2 * size + y)
				end
			end
			return size * self.width, size * self.height
		end

		GLYPHS[char] = glyph
	end

	local serialized = {'GLYPHS = {\n'}
	for g, data in pairs(GLYPHS) do
		serialized[#serialized+1] = ('\t[%q] = { width = %f, height = %f, '):format(g, data.width, data.height)
		serialized[#serialized+1] = 'vertices = {'
		serialized[#serialized+1] = table.concat(data.vertices, ', ')
		serialized[#serialized+1] = '} },\n'
	end
	serialized[#serialized+1] = '}'
	print(table.concat(serialized))

	function love.graphics.print(str,x,y,size,jitter,times)
		for c in str:upper():gmatch('.') do
			if not GLYPHS[c] then
				x = x + size
			else
				x = x + GLYPHS[c]:draw(x,y,size,jitter,times) + size / 10
			end
		end
	end
	local t = 0 function love.update(dt) t = t + dt end
	function love.draw()
		love.graphics.setColor(255,255,255, 200)
		love.graphics.print('HELLO WORLD!', 30,300, 60, 1 +  math.sin(t/2), 3)
	end
end)

function love.load()
	glyph, char = next_glyph()
	love.graphics.setFont(20)
end

function love.draw()
	love.graphics.print(char, 10,570)

	if #glyph >= 4 then
		love.graphics.setColor(0,255,0)
		love.graphics.setLine(2)
		love.graphics.rectangle('line', get_bbox(glyph))

		love.graphics.setColor(255,255,255)
		love.graphics.setLine(10)
		for i=1,#glyph,4 do
			love.graphics.line(glyph[i], glyph[i+1], glyph[i+2], glyph[i+3])
		end
	end

	love.graphics.setColor(255,255,255)
	love.graphics.setLine(10)
	if x1 and y1 then
		love.graphics.line(x1,y1, love.mouse.getPosition())
	end

	love.graphics.setColor(200,200,200)
	love.graphics.setLine(2)
	local x,y = 5,5
	for _,g in pairs(GLYPHS) do
		x = x + g:draw(x,y,60,.3,5) + 5
		if x > 800 - 60 then
			x = 5
			y = y + 60
		end
	end

end

function love.mousepressed(x,y,btn)
	if btn == 'r' then
		glyph, char = next_glyph()
		return
	end
	if btn == 'l' then
		x1, y1 = x,y
	end
end

function love.mousereleased(x2,y2,btn)
	if btn == 'l' then
		glyph[#glyph+1] = x1
		glyph[#glyph+1] = y1
		glyph[#glyph+1] = x2
		glyph[#glyph+1] = y2
		x1, y1 = nil,nil
	end
end

function love.keypressed(key)
	if key == 'c' then
		for i=#glyph,1 do glyph[i] = nil end
	elseif key == 'd' then
		glyph[#glyph] = nil
		glyph[#glyph] = nil
		glyph[#glyph] = nil
		glyph[#glyph] = nil
	end
end
